"""Shared library-level constants.

This module centralizes canonical string identifiers that are reused across
registration, configuration, and runtime dispatch logic.
"""

# Toolkit registration names
TOOLKIT_ARXIV = "arxiv"
TOOLKIT_AUDIO = "audio"
TOOLKIT_BASH = "bash"
TOOLKIT_DOCUMENT = "document"
TOOLKIT_FILE_EDIT = "file_edit"
TOOLKIT_GITHUB = "github"
TOOLKIT_GMAIL = "gmail"
TOOLKIT_IMAGE = "image"
TOOLKIT_JINA_RESEARCH = "jina_research"
TOOLKIT_MCP = "mcp"
TOOLKIT_MEMORY = "memory"
TOOLKIT_PYTHON_EXECUTOR = "python_executor"
TOOLKIT_SERPER = "serper"
TOOLKIT_TABULAR_DATA = "tabular_data"
TOOLKIT_USER_INTERACTION = "user_interaction"
TOOLKIT_VIDEO = "video"
TOOLKIT_WEB_SEARCH = "web_search"
TOOLKIT_WIKIPEDIA = "wikipedia"

# Built-in subagent names / types
SUBAGENT_ASKURA = "askura"
SUBAGENT_BROWSER_USE = "browser_use"
SUBAGENT_CLAUDE = "claude"
SUBAGENT_TACITUS = "tacitus"
